/**
 * JASMINe
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DroolsWorkingMemory.java 2693 2008-10-30 13:12:22Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.rules.ra.outbound;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.resource.ResourceException;
import javax.resource.cci.Connection;
import javax.resource.cci.ConnectionMetaData;
import javax.resource.cci.Interaction;
import javax.resource.cci.LocalTransaction;
import javax.resource.cci.ResultSetInfo;

import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.compiler.DroolsParserException;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderConfiguration;
import org.drools.rule.Package;
import org.drools.rule.Rule;
import org.drools.rule.builder.dialect.java.JavaDialectConfiguration;
import org.ow2.jasmine.rules.AbstractDroolsWorkingMemory;

/**
 * A service implementation to manage the Drools working memory.
 */
public class DroolsWorkingMemory extends AbstractDroolsWorkingMemory implements Connection {

    /**
     * Logger.
     */
    private static Logger logger = Logger.getLogger(DroolsWorkingMemory.class
            .getName());

    /**
     * Location to get rule definitions.
     */
    private String location = null;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public DroolsWorkingMemory(final ClassLoader classloader) {
        super(classloader, logger);
    }

    // ------------------------------------------------------------------------
    // Implementation of the IDroolsWorkingMemory interface
    // ------------------------------------------------------------------------


    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.api.IDroolsWorkingMemory#start()
     */
    public void start() throws Exception {

        if("linux".equals(System.getProperty("os.name"))){
            logger.log(Level.INFO, "Load the files contained in $JASMINE_RULES");
        } else if("windows".equals(System.getProperty("os.name"))){
            logger.log(Level.INFO, "Load the files contained in %JASMINE_RULES%");
        }

        if (location == null) {
            location = System.getenv("JASMINE_RULES");
            if (location == null) {
                logger
                        .log(Level.SEVERE,
                                "The environment variable JASMINE_RULES does not exist");
                throw new Exception(
                        "The environment variable JASMINE_RULES does not exist");
            }
        }

        ruleBase = loadRules();
        // Create and init a new WorkingMemory
        createWorkingMemory();
        startRuleExecutor();
    }

    // ------------------------------------------------------------------------
    // Implementation of the Connection interface
    // ------------------------------------------------------------------------

    public void close() throws ResourceException {
        // TODO Auto-generated method stub

    }

    public Interaction createInteraction() throws ResourceException {
        // TODO Auto-generated method stub
        return null;
    }

    public LocalTransaction getLocalTransaction() throws ResourceException {
        // TODO Auto-generated method stub
        return null;
    }

    public ConnectionMetaData getMetaData() throws ResourceException {
        // TODO Auto-generated method stub
        return null;
    }

    public ResultSetInfo getResultSetInfo() throws ResourceException {
        // TODO Auto-generated method stub
        return null;
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * Add a file to the WorkingMemory
     *
     * @param bytes
     *            the data of the file
     * @param fileName
     *            the name of the file
     */
    // public void addFileToWorking(final byte[] bytes, final String fileName)
    // throws Exception {
    // // Save data in $JASMINE_RULES/fileName
    // saveData(bytes, fileName);
    //
    // RuleBase ruleBase = loadRules();
    //
    // restoreWorkingMemory(ruleBase);
    // }

    /**
     * Save data in JASMINE_RULES/fileName
     *
     * @param bytes
     *            the data
     * @param fileName
     *            the name of the file
     * @throws Exception
     */
    // private void saveData(final byte[] bytes, final String fileName) throws
    // Exception {
    //
    // logger.log(Level.INFO, "Save data in JASMINE_RULES/" + fileName);
    //
    // String location = System.getenv("JASMINE_RULES");
    // if (location == null) {
    // throw new Exception(
    // "The environment variable JASMINE_RULES does not exist");
    // }
    //
    // File file = new File(location + File.separator + fileName);
    // if (!file.exists()) {
    // file.createNewFile();
    // }
    //
    // FileOutputStream os = new FileOutputStream(file);
    //
    // os.write(bytes);
    // os.close();
    // }
    /**
     * @return the rule base created with rules found at location
     */
    private RuleBase loadRules() {
        ClassLoader lastClassLoader = Thread.currentThread()
                .getContextClassLoader();
        Thread.currentThread().setContextClassLoader(
                getClass().getClassLoader());
        try {
            logger
                    .log(Level.INFO,
                            "Configuration of the packageBuiler for the new ruleBase...");
            PackageBuilderConfiguration pkgBuilderCfg = new PackageBuilderConfiguration(
                    classLoader);
            JavaDialectConfiguration javaConf = (JavaDialectConfiguration) pkgBuilderCfg
                    .getDialectConfiguration("java");
            javaConf.setCompiler(JavaDialectConfiguration.JANINO);
            if (dumpDir != null) {
                pkgBuilderCfg.setDumpDir(dumpDir);
            }
            PackageBuilder packageBuilder = new PackageBuilder(pkgBuilderCfg);
            RuleBase ruleBase = RuleBaseFactory.newRuleBase();

            File directory = new File(location);
            if (directory.isDirectory()) {
                for (File file : directory.listFiles()) {

                    logger.log(Level.INFO, "loading " + file.getAbsolutePath()
                            + " ...");

                    try {

                        FileReader fr = new FileReader(file.getAbsoluteFile());
                        String name = file.getName();

                        if (name.endsWith(".drl.xml")) {
                            packageBuilder.addPackageFromXml(fr);
                            logger.log(Level.INFO, file.getAbsolutePath()
                                    + " loaded");
                        } else if (name.endsWith(".drl")) {
                            File dslFile = new File(name.substring(0, name
                                    .length() - 4)
                                    + ".dsl");
                            if (dslFile.exists()) {
                                packageBuilder.addPackageFromDrl(fr,
                                        new FileReader(dslFile));
                            } else {
                                packageBuilder.addPackageFromDrl(fr);
                            }
                            logger.log(Level.INFO, file.getAbsolutePath()
                                    + " loaded");

                        } else {
                            logger.log(Level.INFO, "extension not recognized");
                        }

                    } catch (DroolsParserException e) {
                        logger.log(Level.WARNING, "Cannot add file "
                                + file.getName() + "to the package builder", e);
                    } catch (FileNotFoundException e) {
                        logger.log(Level.INFO, file.getAbsolutePath()
                                + " ignored");
                        logger.log(Level.FINE, "Cannot access at " + file, e);
                    } catch (IOException e) {
                        logger.log(Level.WARNING, "Cannot add file "
                                + file.getName() + "to the package builder", e);
                    }

                }
            }

            Package pack = null;
            try {
                pack = packageBuilder.getPackage();
                if (pack != null) {
                    pack.checkValidity();
                    Rule[] rules = pack.getRules();
                    if (rules.length != 0) {
                        ruleBase.addPackage(pack);
                    }
                }
            } catch (Exception e) {
                logger.log(Level.SEVERE, "Cannot add a package for " + pack, e);
            }
            return ruleBase;
        } finally {
            Thread.currentThread().setContextClassLoader(lastClassLoader);
        }
    }
}
